@@ -83,7 +83,15 @@ def exists(val):
 # Agent output types
 # agent_output_type = Union[BaseModel, dict, str]
 agent_output_type = Literal[
-    "string", "str", "list", "json", "dict", "yaml", "json_schema"
+    "string",
+    "str",
+    "list",
+    "json",
+    "dict",
+    "yaml",
+    "json_schema",
+    "memory-list",
+    "memory-dict",
 ]
 ToolUsageType = Union[BaseModel, Dict[str, Any]]
 
@@ -341,6 +349,7 @@ class Agent:
         load_state_path: str = None,
         role: agent_roles = "worker",
         no_print: bool = False,
+        tools_list_dictionary: Optional[List[Dict[str, Any]]] = None,
         *args,
         **kwargs,
     ):
@@ -459,11 +468,12 @@ class Agent:
         self.load_state_path = load_state_path
         self.role = role
         self.no_print = no_print
+        self.tools_list_dictionary = tools_list_dictionary
 
         # Initialize the short term memory
         self.short_memory = Conversation(
             system_prompt=system_prompt,
-            time_enabled=True,
+            time_enabled=False,
             user=user_name,
             rules=rules,
             *args,
@@ -578,23 +588,59 @@ class Agent:
         # Telemetry Processor to log agent data
         log_agent_data(self.to_dict())
 
-        if self.llm is None and self.model_name is not None:
+        if self.llm is None:
             self.llm = self.llm_handling()
 
     def llm_handling(self):
         from swarms.utils.litellm_wrapper import LiteLLM
 
-        if self.llm_args is not None:
-            llm = LiteLLM(model_name=self.model_name, **self.llm_args)
-
-        else:
-            llm = LiteLLM(
-                model_name=self.model_name,
-                temperature=self.temperature,
-                max_tokens=self.max_tokens,
+        if self.model_name is None:
+            # raise ValueError("Model name cannot be None")
+            logger.warning(
+                "Model name is not provided, using gpt-4o-mini. You can configure any model from litellm if desired."
             )
+            self.model_name = "gpt-4o-mini"
 
-        return llm
+        try:
+            if self.llm_args is not None:
+                llm = LiteLLM(
+                    model_name=self.model_name, **self.llm_args
+                )
+            elif self.tools_list_dictionary is not None:
+
+                length_of_tools_list_dictionary = len(
+                    self.tools_list_dictionary
+                )
+
+                if length_of_tools_list_dictionary > 0:
+
+                    parallel_tool_calls = True
+
+                llm = LiteLLM(
+                    model_name=self.model_name,
+                    temperature=self.temperature,
+                    max_tokens=self.max_tokens,
+                    system_prompt=self.system_prompt,
+                    tools_list_dictionary=self.tools_list_dictionary,
+                    tool_choice="auto",
+                    parallel_tool_calls=parallel_tool_calls,
+                )
+            else:
+                llm = LiteLLM(
+                    model_name=self.model_name,
+                    temperature=self.temperature,
+                    max_tokens=self.max_tokens,
+                    system_prompt=self.system_prompt,
+                )
+            return llm
+        except Exception as e:
+            logger.error(f"Error in llm_handling: {e}")
+            return None
+
+    def prepare_tools_list_dictionary(self):
+        import json
+
+        return json.loads(self.tools_list_dictionary)
 
     def check_if_no_prompt_then_autogenerate(self, task: str = None):
         """
@@ -1058,13 +1104,13 @@ class Agent:
                 self.short_memory.get_str()
             )
 
-            # Handle artifacts
-            if self.artifacts_on is True:
-                self.handle_artifacts(
-                    concat_strings(all_responses),
-                    self.artifacts_output_path,
-                    self.artifacts_file_extension,
-                )
+            # # Handle artifacts
+            # if self.artifacts_on is True:
+            #     self.handle_artifacts(
+            #         concat_strings(all_responses),
+            #         self.artifacts_output_path,
+            #         self.artifacts_file_extension,
+            # )
 
             log_agent_data(self.to_dict())
             if self.autosave is True:
@@ -1093,6 +1139,14 @@ class Agent:
                 return yaml.safe_dump(
                     self.agent_output.model_dump(), sort_keys=False
                 )
+
+            elif self.output_type == "memory-list":
+                return self.short_memory.return_messages_as_list()
+
+            elif self.output_type == "memory-dict":
+                return (
+                    self.short_memory.return_messages_as_dictionary()
+                )
             elif self.return_history is True:
                 history = self.short_memory.get_str()
 
@@ -1625,7 +1679,7 @@ class Agent:
             ):
                 self.short_memory = Conversation(
                     system_prompt=self.system_prompt,
-                    time_enabled=True,
+                    time_enabled=False,
                     user=self.user_name,
                     rules=self.rules,
                 )
@@ -2407,11 +2461,11 @@ class Agent:
         if not isinstance(task, str):
             raise TypeError("Task must be a string")
 
-        if not task.strip():
-            raise ValueError("Task cannot be empty")
+        if task is None:
+            raise ValueError("Task cannot be None")
 
-        if self.llm is None:
-            raise TypeError("LLM object cannot be None")
+        # if self.llm is None:
+        #     raise TypeError("LLM object cannot be None")
 
         try:
             out = self.llm.run(task, *args, **kwargs)
