@@ -1,11 +1,14 @@
 import datetime
 import json
-from typing import Any, Optional
+from typing import Any, Optional, Union
 
 import yaml
 from swarms.structs.base_structure import BaseStructure
 from typing import TYPE_CHECKING
+from swarms.utils.any_to_str import any_to_str
 from swarms.utils.formatter import formatter
+from swarms.utils.litellm_tokenizer import count_tokens
+import threading
 
 if TYPE_CHECKING:
     from swarms.structs.agent import (
@@ -15,53 +18,25 @@ if TYPE_CHECKING:
 
 class Conversation(BaseStructure):
     """
-    A class structure to represent a conversation in a chatbot. This class is used to store the conversation history.
-    And, it can be used to save the conversation history to a file, load the conversation history from a file, and
-    display the conversation history. We can also use this class to add the conversation history to a database, query
-    the conversation history from a database, delete the conversation history from a database, update the conversation
-    history from a database, and get the conversation history from a database.
-
-
-    Args:
-        time_enabled (bool): Whether to enable timestamps for the conversation history. Default is False.
-        database (AbstractDatabase): The database to use for storing the conversation history. Default is None.
-        autosave (bool): Whether to autosave the conversation history to a file. Default is None.
-        save_filepath (str): The filepath to save the conversation history to. Default is None.
-
-
-    Methods:
-        add(role: str, content: str): Add a message to the conversation history.
-        delete(index: str): Delete a message from the conversation history.
-        update(index: str, role, content): Update a message in the conversation history.
-        query(index: str): Query a message in the conversation history.
-        search(keyword: str): Search for a message in the conversation history.
-        display_conversation(detailed: bool = False): Display the conversation history.
-        export_conversation(filename: str): Export the conversation history to a file.
-        import_conversation(filename: str): Import a conversation history from a file.
-        count_messages_by_role(): Count the number of messages by role.
-        return_history_as_string(): Return the conversation history as a string.
-        save_as_json(filename: str): Save the conversation history as a JSON file.
-        load_from_json(filename: str): Load the conversation history from a JSON file.
-        search_keyword_in_conversation(keyword: str): Search for a keyword in the conversation history.
-        pretty_print_conversation(messages): Pretty print the conversation history.
-        add_to_database(): Add the conversation history to the database.
-        query_from_database(query): Query the conversation history from the database.
-        delete_from_database(): Delete the conversation history from the database.
-        update_from_database(): Update the conversation history from the database.
-        get_from_database(): Get the conversation history from the database.
-        execute_query_from_database(query): Execute a query on the database.
-        fetch_all_from_database(): Fetch all from the database.
-        fetch_one_from_database(): Fetch one from the database.
-
-    Examples:
-        >>> from swarms import Conversation
-        >>> conversation = Conversation()
-        >>> conversation.add("user", "Hello, how are you?")
-        >>> conversation.add("assistant", "I am doing well, thanks.")
-        >>> conversation.display_conversation()
-        user: Hello, how are you?
-        assistant: I am doing well, thanks.
+    A class to manage a conversation history, allowing for the addition, deletion,
+    and retrieval of messages, as well as saving and loading the conversation
+    history in various formats.
 
+    Attributes:
+        system_prompt (Optional[str]): The system prompt for the conversation.
+        time_enabled (bool): Flag to enable time tracking for messages.
+        autosave (bool): Flag to enable automatic saving of conversation history.
+        save_filepath (str): File path for saving the conversation history.
+        tokenizer (Any): Tokenizer for counting tokens in messages.
+        context_length (int): Maximum number of tokens allowed in the conversation history.
+        rules (str): Rules for the conversation.
+        custom_rules_prompt (str): Custom prompt for rules.
+        user (str): The user identifier for messages.
+        auto_save (bool): Flag to enable auto-saving of conversation history.
+        save_as_yaml (bool): Flag to save conversation history as YAML.
+        save_as_json_bool (bool): Flag to save conversation history as JSON.
+        token_count (bool): Flag to enable token counting for messages.
+        conversation_history (list): List to store the history of messages.
     """
 
     def __init__(
@@ -78,9 +53,28 @@ class Conversation(BaseStructure):
         auto_save: bool = True,
         save_as_yaml: bool = True,
         save_as_json_bool: bool = False,
+        token_count: bool = True,
         *args,
         **kwargs,
     ):
+        """
+        Initializes the Conversation object with the provided parameters.
+
+        Args:
+            system_prompt (Optional[str]): The system prompt for the conversation.
+            time_enabled (bool): Flag to enable time tracking for messages.
+            autosave (bool): Flag to enable automatic saving of conversation history.
+            save_filepath (str): File path for saving the conversation history.
+            tokenizer (Any): Tokenizer for counting tokens in messages.
+            context_length (int): Maximum number of tokens allowed in the conversation history.
+            rules (str): Rules for the conversation.
+            custom_rules_prompt (str): Custom prompt for rules.
+            user (str): The user identifier for messages.
+            auto_save (bool): Flag to enable auto-saving of conversation history.
+            save_as_yaml (bool): Flag to save conversation history as YAML.
+            save_as_json_bool (bool): Flag to save conversation history as JSON.
+            token_count (bool): Flag to enable token counting for messages.
+        """
         super().__init__()
         self.system_prompt = system_prompt
         self.time_enabled = time_enabled
@@ -95,6 +89,7 @@ class Conversation(BaseStructure):
         self.auto_save = auto_save
         self.save_as_yaml = save_as_yaml
         self.save_as_json_bool = save_as_json_bool
+        self.token_count = token_count
 
         # If system prompt is not None, add it to the conversation history
         if self.system_prompt is not None:
@@ -110,48 +105,75 @@ class Conversation(BaseStructure):
         if tokenizer is not None:
             self.truncate_memory_with_tokenizer()
 
-    def add(self, role: str, content: str, *args, **kwargs):
-        """Add a message to the conversation history
+    def add(
+        self,
+        role: str,
+        content: Union[str, dict, list],
+        *args,
+        **kwargs,
+    ):
+        """Add a message to the conversation history.
 
         Args:
-            role (str): The role of the speaker
-            content (str): The content of the message
-
+            role (str): The role of the speaker (e.g., 'User', 'System').
+            content (Union[str, dict, list]): The content of the message to be added.
         """
-        if self.time_enabled:
-            now = datetime.datetime.now()
-            timestamp = now.strftime("%Y-%m-%d %H:%M:%S")
-            message = {
-                "role": role,
-                "content": content,
-                "timestamp": timestamp,
-            }
-        else:
-            message = {
-                "role": role,
-                "content": content,
-            }
+        now = datetime.datetime.now()
+        timestamp = now.strftime("%Y-%m-%d %H:%M:%S")
 
+        # Base message with role
+        message = {
+            "role": role,
+        }
+
+        # Handle different content types
+        if isinstance(content, dict) or isinstance(content, list):
+            message["content"] = content
+        else:
+            message["content"] = f"Time: {timestamp} \n {content}"
+
+        # Add the message to history immediately without waiting for token count
         self.conversation_history.append(message)
 
-        if self.autosave:
+        # If token counting is enabled, do it in a separate thread
+        if self.token_count is True:
+            # Define a function to count tokens and update the message
+            def count_tokens_thread():
+                tokens = count_tokens(any_to_str(content))
+                # Update the message that's already in the conversation history
+                message["token_count"] = int(tokens)
+
+                # If autosave is enabled, save after token count is updated
+                if self.autosave:
+                    self.save_as_json(self.save_filepath)
+
+            # Start a new thread for token counting
+            token_thread = threading.Thread(
+                target=count_tokens_thread
+            )
+            token_thread.daemon = (
+                True  # Make thread terminate when main program exits
+            )
+            token_thread.start()
+        elif self.autosave:
+            # If token counting is disabled but autosave is enabled, save immediately
             self.save_as_json(self.save_filepath)
 
     def delete(self, index: str):
-        """Delete a message from the conversation history
+        """Delete a message from the conversation history.
 
         Args:
-            index (str): index of the message to delete
+            index (str): Index of the message to delete.
         """
         self.conversation_history.pop(index)
 
     def update(self, index: str, role, content):
-        """Update a message in the conversation history
+        """Update a message in the conversation history.
 
         Args:
-            index (str): index of the message to update
-            role (_type_): role of the speaker
-            content (_type_): content of the message
+            index (str): Index of the message to update.
+            role (str): Role of the speaker.
+            content (Union[str, dict]): New content of the message.
         """
         self.conversation_history[index] = {
             "role": role,
@@ -159,24 +181,24 @@ class Conversation(BaseStructure):
         }
 
     def query(self, index: str):
-        """Query a message in the conversation history
+        """Query a message in the conversation history.
 
         Args:
-            index (str): index of the message to query
+            index (str): Index of the message to query.
 
         Returns:
-            str: the message
+            dict: The message with its role and content.
         """
         return self.conversation_history[index]
 
     def search(self, keyword: str):
-        """Search for a message in the conversation history
+        """Search for a message in the conversation history.
 
         Args:
-            keyword (str): Keyword to search for
+            keyword (str): Keyword to search for.
 
         Returns:
-            str: description
+            list: List of messages containing the keyword.
         """
         return [
             msg
@@ -185,10 +207,10 @@ class Conversation(BaseStructure):
         ]
 
     def display_conversation(self, detailed: bool = False):
-        """Display the conversation history
+        """Display the conversation history.
 
         Args:
-            detailed (bool, optional): detailed. Defaults to False.
+            detailed (bool, optional): Flag to display detailed information. Defaults to False.
         """
         for message in self.conversation_history:
             formatter.print_panel(
@@ -196,20 +218,20 @@ class Conversation(BaseStructure):
             )
 
     def export_conversation(self, filename: str, *args, **kwargs):
-        """Export the conversation history to a file
+        """Export the conversation history to a file.
 
         Args:
-            filename (str): filename to export to
+            filename (str): Filename to export to.
         """
         with open(filename, "w") as f:
             for message in self.conversation_history:
                 f.write(f"{message['role']}: {message['content']}\n")
 
     def import_conversation(self, filename: str):
-        """Import a conversation history from a file
+        """Import a conversation history from a file.
 
         Args:
-            filename (str): filename to import from
+            filename (str): Filename to import from.
         """
         with open(filename) as f:
             for line in f:
@@ -217,7 +239,11 @@ class Conversation(BaseStructure):
                 self.add(role, content.strip())
 
     def count_messages_by_role(self):
-        """Count the number of messages by role"""
+        """Count the number of messages by role.
+
+        Returns:
+            dict: A dictionary with counts of messages by role.
+        """
         counts = {
             "system": 0,
             "user": 0,
@@ -229,10 +255,10 @@ class Conversation(BaseStructure):
         return counts
 
     def return_history_as_string(self):
-        """Return the conversation history as a string
+        """Return the conversation history as a string.
 
         Returns:
-            str: the conversation history
+            str: The conversation history formatted as a string.
         """
         return "\n".join(
             [
@@ -242,39 +268,41 @@ class Conversation(BaseStructure):
         )
 
     def get_str(self):
+        """Get the conversation history as a string.
+
+        Returns:
+            str: The conversation history.
+        """
         return self.return_history_as_string()
 
     def save_as_json(self, filename: str = None):
-        """Save the conversation history as a JSON file
+        """Save the conversation history as a JSON file.
 
         Args:
-            filename (str): Save the conversation history as a JSON file
+            filename (str): Filename to save the conversation history.
         """
-        # Create the directory if it does not exist
-        # os.makedirs(os.path.dirname(filename), exist_ok=True)
         if filename is not None:
             with open(filename, "w") as f:
                 json.dump(self.conversation_history, f)
 
     def load_from_json(self, filename: str):
-        """Load the conversation history from a JSON file
+        """Load the conversation history from a JSON file.
 
         Args:
-            filename (str): filename to load from
+            filename (str): Filename to load from.
         """
-        # Load the conversation history from a JSON file
         if filename is not None:
             with open(filename) as f:
                 self.conversation_history = json.load(f)
 
     def search_keyword_in_conversation(self, keyword: str):
-        """Search for a keyword in the conversation history
+        """Search for a keyword in the conversation history.
 
         Args:
-            keyword (str): keyword to search for
+            keyword (str): Keyword to search for.
 
         Returns:
-            str: description
+            list: List of messages containing the keyword.
         """
         return [
             msg
@@ -283,10 +311,10 @@ class Conversation(BaseStructure):
         ]
 
     def pretty_print_conversation(self, messages):
-        """Pretty print the conversation history
+        """Pretty print the conversation history.
 
         Args:
-            messages (str): messages to print
+            messages (list): List of messages to print.
         """
         role_to_color = {
             "system": "red",
@@ -367,15 +395,31 @@ class Conversation(BaseStructure):
         self.conversation_history = truncated_history
 
     def clear(self):
+        """Clear the conversation history."""
         self.conversation_history = []
 
     def to_json(self):
+        """Convert the conversation history to a JSON string.
+
+        Returns:
+            str: The conversation history as a JSON string.
+        """
         return json.dumps(self.conversation_history)
 
     def to_dict(self):
+        """Convert the conversation history to a dictionary.
+
+        Returns:
+            list: The conversation history as a list of dictionaries.
+        """
         return self.conversation_history
 
     def to_yaml(self):
+        """Convert the conversation history to a YAML string.
+
+        Returns:
+            str: The conversation history as a YAML string.
+        """
         return yaml.dump(self.conversation_history)
 
     def get_visible_messages(self, agent: "Agent", turn: int):
@@ -406,23 +450,80 @@ class Conversation(BaseStructure):
         return visible_messages
 
     def get_last_message_as_string(self):
-        # fetch the last message from the conversation history with the agent name and the message of the agent
+        """Fetch the last message from the conversation history.
+
+        Returns:
+            str: The last message formatted as 'role: content'.
+        """
         return f"{self.conversation_history[-1]['role']}: {self.conversation_history[-1]['content']}"
 
     def return_messages_as_list(self):
-        # we must concat the role and the content of the message
+        """Return the conversation messages as a list of formatted strings.
+
+        Returns:
+            list: List of messages formatted as 'role: content'.
+        """
         return [
             f"{message['role']}: {message['content']}"
             for message in self.conversation_history
         ]
 
+    def return_messages_as_dictionary(self):
+        """Return the conversation messages as a list of dictionaries.
 
-# Example usage
-conversation = Conversation()
-conversation.add("user", "Hello, how are you?")
-# print(conversation.get_last_message_as_string())
-# print(conversation.return_messages_as_list())
+        Returns:
+            list: List of dictionaries containing role and content of each message.
+        """
+        return [
+            {
+                "role": message["role"],
+                "content": message["content"],
+            }
+            for message in self.conversation_history
+        ]
+
+    def add_tool_output_to_agent(self, role: str, tool_output: dict):
+        """
+        Add a tool output to the conversation history.
+
+        Args:
+            role (str): The role of the tool.
+            tool_output (dict): The output from the tool to be added.
+        """
+        self.add(role, tool_output)
+
+    def return_json(self):
+        """Return the conversation messages as a JSON string.
+
+        Returns:
+            str: The conversation messages formatted as a JSON string.
+        """
+        return json.dumps(
+            self.return_messages_as_dictionary(), indent=4
+        )
+
+    def get_final_message(self):
+        """Return the final message from the conversation history.
+
+        Returns:
+            str: The final message formatted as 'role: content'.
+        """
+        return f"{self.conversation_history[-1]['role']}: {self.conversation_history[-1]['content']}"
+
+
+# # Example usage
+# # conversation = Conversation()
+# conversation = Conversation(token_count=True)
+# conversation.add("user", "Hello, how are you?")
 # conversation.add("assistant", "I am doing well, thanks.")
-# # print(conversation.to_json())
-# print(type(conversation.to_dict()))
+# # conversation.add(
+# #     "assistant", {"name": "tool_1", "output": "Hello, how are you?"}
+# # )
+# # print(conversation.return_json())
+
+# # # print(conversation.get_last_message_as_string())
+# print(conversation.return_json())
+# # # conversation.add("assistant", "I am doing well, thanks.")
+# # # # print(conversation.to_json())
+# # print(type(conversation.to_dict()))
 # # print(conversation.to_yaml())
