@@ -18,7 +18,9 @@ from swarms.structs.rearrange import AgentRearrange
 from swarms.structs.sequential_workflow import SequentialWorkflow
 from swarms.structs.spreadsheet_swarm import SpreadSheetSwarm
 from swarms.structs.swarm_matcher import swarm_matcher
+from swarms.structs.output_type import OutputType
 from swarms.utils.loguru_logger import initialize_logger
+from swarms.structs.malt import MALT
 
 logger = initialize_logger(log_folder="swarm_router")
 
@@ -34,6 +36,7 @@ SwarmType = Literal[
     "HiearchicalSwarm",
     "auto",
     "MajorityVoting",
+    "MALT",
 ]
 
 
@@ -146,7 +149,7 @@ class SwarmRouter:
         shared_memory_system: Any = None,
         rules: str = None,
         documents: List[str] = [],  # A list of docs file paths
-        output_type: str = "all",
+        output_type: OutputType = "dict",
         no_cluster_ops: bool = False,
         speaker_fn: callable = None,
         load_agents_from_csv: bool = False,
@@ -304,14 +307,24 @@ class SwarmRouter:
                 **kwargs,
             )
 
+        elif self.swarm_type == "MALT":
+            return MALT(
+                name=self.name,
+                description=self.description,
+                max_loops=self.max_loops,
+                return_dict=True,
+                preset_agents=True,
+            )
+
         elif self.swarm_type == "HiearchicalSwarm":
             return HierarchicalSwarm(
                 name=self.name,
                 description=self.description,
-                director=self.agents[0],
+                # director=self.agents[0],
                 agents=self.agents,
                 max_loops=self.max_loops,
                 return_all_history=self.return_entire_history,
+                output_type=self.output_type,
                 *args,
                 **kwargs,
             )
@@ -323,6 +336,7 @@ class SwarmRouter:
                 aggregator_system_prompt=aggregator_system_prompt.get_prompt(),
                 aggregator_agent=self.agents[-1],
                 layers=self.max_loops,
+                output_type=self.output_type,
                 *args,
                 **kwargs,
             )
@@ -386,6 +400,7 @@ class SwarmRouter:
                 max_loops=self.max_loops,
                 auto_save=self.autosave,
                 return_str_on=self.return_entire_history,
+                output_type=self.output_type,
                 *args,
                 **kwargs,
             )
@@ -438,18 +453,12 @@ class SwarmRouter:
         self.swarm = self._create_swarm(task, *args, **kwargs)
 
         try:
-            # self._log(
-            #     "info",
-            #     f"Running task on {self.swarm_type} swarm with task: {task}",
-            # )
+            logger.info(
+                f"Running task on {self.swarm_type} swarm with task: {task}"
+            )
             result = self.swarm.run(task=task, *args, **kwargs)
 
-            # self._log(
-            #     "success",
-            #     f"Task completed successfully on {self.swarm_type} swarm",
-            #     task=task,
-            #     metadata={"result": str(result)},
-            # )
+            logger.info("Swarm completed successfully")
             return result
         except Exception as e:
             self._log(
